/*
 * Muster_CAN.c
 *
 * Created: 18.03.2013 14:50:09
 *  Author: A. Lbke
 */ 


#include <avr/io.h>
#include <avr/interrupt.h>	/* Interrupts einbinden */

#include "my_io.h"			/* Controller IO einbinden */
#include "my_CAN.h"			/* CAN-Funktionen einbinden */

/* Globale Variablen */
volatile uint8_t but_freigabe = 1;	/* Freigabe fr die Taster (1=freigegeben, 0=gesperrt) */
volatile uint8_t but_ticks	  = 31;	/* Entprellzeit fr die Taster*/

/**** Tasterbelegung ****/	/* Tasterbelegung des Evaluationsboards */
#define t_NORTH		PE4		/* oben -> LED 1*/
#define t_SOUTH		PE7     /* unten -> LED 3*/
#define t_WEST		PE6     /* links -> LED 4*/
#define t_EAST		PE5		/* rechts -> LED 2*/
#define t_CENTER	PE2     /* reset / bitmuster */


/* ISR (BADISR) */
ISR(BADISR_vect)		/* ISR zum Abfangen von nicht gewollten Interrupts */
{
}

/**** Hauptroutine *************************************/
int main(void) {


	/**** Initialisierungen ****/
	init_ports();		/* Ports initialisieren */
	init_timer0();		/* Timer0 fr Zeitbasis initialisieren */
	can_init();			/* CAN-Schnittstelle initialisieren */

	sei();				/* Interrupts global freigeben */

	/**** Variablen *******/
	// In dieser Variable wird der Status der LEDs auf dem anderen Board gespeichert
	// Sobald der Taster bettigt wird, wird der Status invertiert (0->1 oder 1->0) und an das 
	// andere Board bertragen
	uint8_t status = 0;	
	
	//Can Nachrichten anlegen
	CAN_message t1_msg;	// LED 1 status
	CAN_message t2_msg; // LED 2 status
	CAN_message t3_msg; // LED 3 status
	CAN_message t4_msg; // LED 4 status
	CAN_message t5_msg;	// LED reset 
			
	/**** Hauptroutine ****/
	while(1) 
	{								
		/* neuen status auslesen und can nachrichten empfangen */
		status = CAN_getMsg(status);				
 		
		// Taster abfragen, Taster Status invertieren, Can Nachricht mit neuem Status erstellen und ber den MOB 0, MOB 1, MOB 2 oder MOB 3 versenden
		// Die MOBS mssen unter my_can.c -> Mob_init angepasst werden			
		if (getTaster(t_NORTH)) { toggleBit(status,0); t1_msg.data[0] = getBitValue(status,0); can_tx (t1_msg, 0); }	// B1:T1 -> B2:LED1 | B2:T1 -> B1:LED1 | StatusLed 0
		if (getTaster(t_EAST) ) { toggleBit(status,1); t2_msg.data[0] = getBitValue(status,1); can_tx (t2_msg, 1); }	// B1:T2 -> B2:LED2 | B2:T2 -> B1:LED2 | StatusLed 1
		if (getTaster(t_SOUTH)) { toggleBit(status,2); t3_msg.data[0] = getBitValue(status,2); can_tx (t3_msg, 2); }	// B1:T3 -> B2:LED3 | B2:T3 -> B1:LED3 | StatusLed 2
		if (getTaster(t_WEST) ) { toggleBit(status,3); t4_msg.data[0] = getBitValue(status,3); can_tx (t4_msg, 3); }	// B1:T4 -> B2:LED4 | B2:T4 -> B1:LED4 | StatusLed 3	
						
		//Taster abfragen und Status LEDs lschen (reset Funktion)
		if(getTaster(t_CENTER)) 
		{			
			// CAN Nachricht mit dem Inhalt (0x00) erstellen und ber MOB 4 versenden
			t5_msg.data[0] = 0;  can_tx (t5_msg, 4);				
			// Taster Status zurcksetzen, sodass die LED beim nchsten drcken eingeschaltet werden kann				
			status = 0;
		}
	} 
		
} /* void main() */