/************************************************************************/
/* my_can.c                                                             */
/************************************************************************/
/* Musterlsung zum Praktikumsversuch CAN								*/
/*																		*/
/* Created:		16.07.2013 (SS2013)										*/
/* Author:		Nils Feldkmper											*/
/*				Jaron Martinez											*/
/************************************************************************/

/************************************************************************/
/* Includes                                                             */
/************************************************************************/
#include <avr/io.h>			// IO Register einbinden
#include <avr/interrupt.h>	// Interrupts einbinden
#include <inttypes.h>		// Zahlentypen einbinden
#include "my_io.h"			// Controller IO einbinden
#include "my_can.h"			// CAN-Funktionen einbinden

/************************************************************************/
/* Defines                                                              */
/************************************************************************/
#define READ_MASK	0xffff
#define WRITE_MASK	0x0000

/************************************************************************/
/* Funktionen                                                           */
/************************************************************************/

// **** Initialisiert nur die bentigten MOb's ****
void MOB_init()															
{	
	// Die Sende- und Empfangsadresse werden in der my_can.h eingestellt. Jedes Board bekommt seine eigenen Adressen
	// z.B:
	// ============
	// BOARD 1
	// ============	
	// #define _BOARD_ADDR_SEND_OFFSET		0x010
	// #define _BOARD_ADDR_GET_OFFSET		0x020
	// 
	// nun verknpft die ID_SEND Funktion und die ID_GET Funktion Byteweise "oder" die Board Adressen mit den Adressen die bergeben werden
	//
	// ID_SEND(0x001) liefert Adresse 0x011
	// ID_SEND(0x003) liefert Adresse 0x013
	//
	// ID_GET(0x001)  liefert Adresse 0x021
	// ID_GET(0x009)  liefert Adresse 0x026
	//
	// ============
	// BOARD 2
	// ============
	// #define _BOARD_ADDR_SEND_OFFSET		0x020
	// #define _BOARD_ADDR_GET_OFFSET		0x010
	//
	// hier sind die Adressen zueinander verdreht, sodass die Adresse zu den Empfangs- und Sendeadressen von Board 1 passen
	//
	// ID_SEND(0x001) liefert Adresse 0x021
	// ID_SEND(0x002) liefert Adresse 0x022
	//
	// ID_GET(0x004)  liefert Adresse 0x014
	
	
	//++++++++++++++++++++++																			
	//senden
	//++++++++++++++++++++++																			
	
	//Senden LED 1 => 0xXX1
	MOB_all[0].id			= ID_SEND(0x001);	// ID
	MOB_all[0].mask			= WRITE_MASK;		// Maske
	MOB_all[0].length		= 1;				// Datenlnge
	MOB_all[0].direction	= 1;				// Richtung, aus(0),senden(1),empfangen(2),frame buffer empfang(3)
	
	//Senden LED 2 => 0xXX2
	MOB_all[1].id			= ID_SEND(0x002);	// ID
	MOB_all[1].mask			= WRITE_MASK;		// Maske
	MOB_all[1].length		= 1;				// Datenlnge
	MOB_all[1].direction	= 1;				// Richtung, aus(0),senden(1),empfangen(2),frame buffer empfang(3)

	//Senden LED 3 => 0xXX3
	MOB_all[2].id			= ID_SEND(0x003);	// ID
	MOB_all[2].mask			= WRITE_MASK;		// Maske
	MOB_all[2].length		= 1;				// Datenlnge
	MOB_all[2].direction	= 1;				// Richtung, aus(0),senden(1),empfangen(2),frame buffer empfang(3)
	
	//Senden LED 4 => 0xXX4
	MOB_all[3].id			= ID_SEND(0x004);	// ID
	MOB_all[3].mask			= WRITE_MASK;		// Maske
	MOB_all[3].length		= 1;				// Datenlnge
	MOB_all[3].direction	= 1;				// Richtung, aus(0),senden(1),empfangen(2),frame buffer empfang(3)

	//Senden Reset LED => 0xXX5
	MOB_all[4].id			= ID_SEND(0x005);	// ID
	MOB_all[4].mask			= WRITE_MASK;		// Maske
	MOB_all[4].length		= 1;				// Datenlnge
	MOB_all[4].direction	= 1;				// Richtung, aus(0),senden(1),empfangen(2),frame buffer empfang(3)

	//++++++++++++++++++++++																			
	//empfangen
	//++++++++++++++++++++++																			

	//Empfangen LED 1 => 0xXX1	
	MOB_all[7].id			= ID_GET(0x001);	// ID
	MOB_all[7].mask			= READ_MASK;		// Maske
	MOB_all[7].length		= 1;				// Datenlnge
	MOB_all[7].direction	= 2;				// Richtung, aus(0),senden(1),empfangen(2),frame buffer empfang(3)							

	//Empfangen LED 2 => 0xXX2
	MOB_all[8].id			= ID_GET(0x002);	// ID
	MOB_all[8].mask			= READ_MASK;		// Maske
	MOB_all[8].length		= 1;				// Datenlnge
	MOB_all[8].direction	= 2;				// Richtung, aus(0),senden(1),empfangen(2),frame buffer empfang(3)
	
	//Empfangen LED 3 => 0xXX3
	MOB_all[9].id			= ID_GET(0x003);	// ID
	MOB_all[9].mask			= READ_MASK;		// Maske
	MOB_all[9].length		= 1;				// Datenlnge
	MOB_all[9].direction	= 2;				// Richtung, aus(0),senden(1),empfangen(2),frame buffer empfang(3)

	//Empfangen LED 4 => 0xXX4
	MOB_all[10].id			= ID_GET(0x004);	// ID
	MOB_all[10].mask		= READ_MASK;		// Maske
	MOB_all[10].length		= 1;				// Datenlnge
	MOB_all[10].direction	= 2;				// Richtung, aus(0),senden(1),empfangen(2),frame buffer empfang(3)

	//Empfangen Reset LED => 0xXX5
	MOB_all[11].id			= ID_GET(0x005);	// ID
	MOB_all[11].mask		= READ_MASK;		// Maske
	MOB_all[11].length		= 1;				// Datenlnge
	MOB_all[11].direction	= 2;				// Richtung, aus(0),senden(1),empfangen(2),frame buffer empfang(3)
	
	//Empfangen Bitmuster vom PC => 0xXX5
	MOB_all[13].id			= ID_GET(0x006);	// ID
	MOB_all[13].mask		= READ_MASK;		// Maske
	MOB_all[13].length		= 1;				// Datenlnge
	MOB_all[13].direction	= 2;				// Richtung, aus(0),senden(1),empfangen(2),frame buffer empfang(3)
}


// **** Die einzelnen MObs mit der Hilfe der CANPAGE auswhlen und konfigurieren.  ****
void MOB_Config()														
{
	for(uint8_t i=0;i<=14;i++)
	{
		CANPAGE = (i<<4);						// Stelle CANPAGE auf i ein, um diesen Konfiguieren zu knnen
		
		// CAN MOb Control and DLC Register		// Konfiguriere jede einzelne MOb
		CANCDMOB |= (MOB_all[i].direction<<6);	// Richtung einstellen
		CANCDMOB |= (0<<RPLV);					// Automatic reply mode bei "1"
		CANCDMOB |= (0<<IDE);					// Identifier Extension: "0"=11Bit ID, "1"=29Bit ID
		CANCDMOB |= (MOB_all[i].length)&(0x0F);	// Die Datenlnge jeder MOb einstellen.

		// CAN Identifier Tag Register			// Stellt die ID jeder MOb ein (Nur fr 11Bit ID!!)
		CANIDT4 = 0x00;
		CANIDT3 = 0x00;
		CANIDT2 = (unsigned char) ((MOB_all[i].id<<5) & 0xE0);					
		CANIDT1 = (unsigned char) (MOB_all[i].id>>3);
	
		// CAN Identifier Mask Register			// Stellt die Maske jeder MOb ein (Nur fr 11Bit ID!!)
		CANIDM4 = 0x00;
		CANIDM3 = 0x00;
		CANIDM2 = (unsigned char) ((MOB_all[i].mask<<5) & 0xE0);					
		CANIDM1 = (unsigned char) (MOB_all[i].mask>>3);		
	}
}


// **** getmob() ****
// **** Get idex of MOb for which an interrupt was detected. ****/
uint8_t getmob( uint32_t bitmask) 										
{
	// Die MOb-Nummer herausfinden
	uint8_t mob;

	if( bitmask==0) 
		return NOMOB;

	 //the bitmask is shifted to the rigth until the LSB is one
	 //the number of such shift operations is the index of the MOb
		for( mob=0; (bitmask & 0x01)==0; bitmask >>= 1, ++mob);

	 //security check
	if ( mob > 14 )
		return NOMOB;
	else 
	 	return mob;
}     


// **** CAN Receive Interrupt Service Routine ****
ISR(CANIT_vect)
{
	uint8_t i, save_canpage, mob;
	uint16_t CAN_Status_Reg;
	CAN_message message;		
 	
	save_canpage = CANPAGE;						// CANPAGE speichern
 	
	CAN_Status_Reg = CANSIT2 | (CANSIT1<<8);
	mob = getmob(CAN_Status_Reg);				// Welcher MOb hat den Interrupt ausgelst?
    
	if( mob == NOMOB )							// Ist mob>14 dann wird aus dieser Funktion herausgegangen.
		return;
    	
	CANPAGE = mob << 4;							// MOb auswhlen, die den Interrupt ausgelst hat.
    
	for (i=0; i<8; i++)							// Die Daten aus der MOb heraus holen und abspeichern.
		message.data[i] = CANMSG;
	
	message.id = 0;								// get ID of selected MOb (Fr 11Bit und 29 Bit ID)
	
	message.id |= (((uint16_t)CANIDT2)>>5)&(0x07);	// 11Bit-ID lesen 
	message.id |= (((uint16_t)CANIDT1)<<3);
		
	message.new_data = 0x01;
	
    message_RX_global = message;				// Die empfangene Nachricht wird global abgespeichert.

	CANSTMOB &= ~(1<<RXOK);						// reset INT reason
    
	CANCDMOB |= (0<<IDE);						// re-enable RX, reconfigure MOb IDE=1    	
	CANCDMOB |= (1<<CONMOB1);    

    CANPAGE = save_canpage;   					// restore CANPAGE, wegen dem Sendevorgang.
}


// **** CAN initialization ****
void can_init(void)
{
	uint8_t i;
												// set Baud Rate Prescaler for CAN controller bit timing
												// (Seite 264 bis 266)
  	CANBT1 = 0x12 ;								// 0x06 = 250kbit/s; 0x12 = 100kbit/s
 												// set propagation time segment (Seite 264 bis 266)
  	CANBT2 = 0x04;
 												// set PHaseSegment2 and PHaseSegment1 (Seite 264 bis 266)
  	CANBT3 = 0x13;
       
  												// for all Message Objects: set the StatusRegister CANSTMOB to zero;
												// Disable MOb CANCDMOB=0, (CONMOB1:0=00)
  	for (i=0; i<NO_MOBS; i++)					// alle MOb's (0 bis 14) einmal durchgehen.
 	{
  	   CANPAGE  = (i<<4);						// MOb auswhlen
  	   CANSTMOB = 0;							// Status Register von dem ausgewhlten MOb lschen.		
 	   CANCDMOB = 0;							// Control Register von dem ausgewhlten MOb lschen.						
 	}
  	MOB_init();									// Rufe Initialisierung der MObs auf
	MOB_Config();								// Konfiguriert die einzelnen MOb's
  
  	// CAN General Interrupt Enable Register
  	CANGIE |= (1<<ENIT); 						// Enable all Interrupts
	CANGIE |= (0<<ENBOFF);						// Enable Bus Off Interrupt
	CANGIE |= (1<<ENRX);						// Enable Receive Interrupt
	CANGIE |= (0<<ENTX);						// Enable Transmit Interrupt
	CANGIE |= (0<<ENERR);						// Enable MOb Errors Interrupt
	CANGIE |= (0<<ENBX);						// Enable Frame Buffer Interrupt
	CANGIE |= (0<<ENERG);						// Enable General Errors Interrupt
	CANGIE |= (0<<ENOVRT);						// Enable CAN Timer Overrum Interrupt

  	// CAN Enable Interrupt MOb Registers
	// Die Interrupts mssen hier gegebenfalls angepasst werden
	// Es knnen nur nachrichten auf den Mobs empfangen werden, wenn der Interrupt fr den Mob aktiviert ist   
  	CANIE2 |= (0<<IEMOB0);							// Set MOB for which Interrupts are enabled
	CANIE2 |= (0<<IEMOB1);	
	CANIE2 |= (0<<IEMOB2);
	CANIE2 |= (0<<IEMOB3);
	CANIE2 |= (0<<IEMOB4);
	CANIE2 |= (0<<IEMOB5);
	CANIE2 |= (0<<IEMOB6);
	CANIE2 |= (1<<IEMOB7);
	
	CANIE1 |= (1<<IEMOB8);						// Set MOB for which Interrupts are enabled
	CANIE1 |= (1<<IEMOB9);	
	CANIE1 |= (1<<IEMOB10);
	CANIE1 |= (1<<IEMOB11);
	CANIE1 |= (0<<IEMOB12);
	CANIE1 |= (1<<IEMOB13);
	CANIE1 |= (0<<IEMOB14);
 
  	// CAN General Control Register
  	CANGCON |= (1<<ENASTB); 					// Set CAN-driver in enable mode (not standby-mode) 	

  	while (!(CANGSTA & (1<<ENFG)));				// wait until module is ready EnableFlag is on.

}//can_init



// **** CAN transmission via mailbox ****
void can_tx (CAN_message msg, uint8_t mob)
{	
	CANPAGE = (mob<<4);							// Stelle CANPAGE auf mob ein, um diesen Konfiguieren zu knnen
	
	for (uint8_t i=0; i<8; i++)					// bergebe die zu sendenden Daten ins Datenregister, CANMSG wird automatisch inkrementiert							
		CANMSG = msg.data [i];
		
	CANCDMOB |= (1<<CONMOB0);					// Freigabe zur bertragung
	
	while (!(CANSTMOB & (1<<TXOK)));			// warte, bis bertragung abgeschlossen ist

	CANSTMOB &= ~(1<<TXOK);						// Setzt TXOK Flag zurck

}


// **** CAN recieve ****
CAN_message* get_RX_message(void)
{
	return &message_RX_global;
}


/**** CAN_getMsg() ****/
uint8_t CAN_getMsg(uint8_t status) 
{	
	CAN_message* message;						// CAN Nachricht anlegen	
	message = get_RX_message();					// Nachricht lesen und in Variable speichern										

	if(message->new_data==1)					// Ist eine neue Nachricht eingetroffen?
	{
		switch(message->id)						// Welche ID hat die Nachricht? Im Anschluss das entsprechende ausfhren.
		{
			// Nachrichten auswerten die durch Tastendrcke auf dem anderen Board
			// ausgelst werden
			case ID_GET(0x001):					// Taster "oben" auf dem anderen Board
				if(getBitValue(message->data[0],0)) { setLED(0); } else { clearLED(0); }	
				break;
			case ID_GET(0x002):					// Taster "rechts" auf dem anderen Board
				if(getBitValue(message->data[0],0)) { setLED(1); } else { clearLED(1); }			
				break;	
			case ID_GET(0x003):					// Taster "unten" auf dem anderen Board
				if(getBitValue(message->data[0],0)) { setLED(2); } else { clearLED(2); }		
				break;	
			case ID_GET(0x004):					// Taster "links" auf dem anderen Board
				if(getBitValue(message->data[0],0)) { setLED(3); } else { clearLED(3); }			
				break;
			case ID_GET(0x005):					// Led Reset
				clearLED(0); 
				clearLED(1);
				clearLED(2); 
				clearLED(3);
				status = 0; 												
				break;
			case ID_GET(0x006):					// PC zugriff				
				// 4 Bits aus der Nachricht auslesen (Bit 0, Bit 1, Bit 2, Bit 3) und den Status 
				// auf dem eigenem Board setzen
				// Es wird der Tastendruck simuliert
				writeBit(status,0, getBitValue(message->data[0],0));
				writeBit(status,1, getBitValue(message->data[0],1));
				writeBit(status,2, getBitValue(message->data[0],2));
				writeBit(status,3, getBitValue(message->data[0],3));				
						
				// Vier Nachrichten zum zweiten Board senden, auf dem dann die LEDs mit dem 
				// neuen Status gesetzt werden					
				CAN_message t1_msg; t1_msg.data[0] = getBitValue(status,0); can_tx (t1_msg, 0);
				CAN_message t2_msg; t2_msg.data[0] = getBitValue(status,1); can_tx (t2_msg, 1);
				CAN_message t3_msg; t3_msg.data[0] = getBitValue(status,2); can_tx (t3_msg, 2);
				CAN_message t4_msg; t4_msg.data[0] = getBitValue(status,3); can_tx (t4_msg, 3);							
				break;							
			default:
			break;
		}
	}
	
	message->new_data=0;						// new_data zurcksetzen 
	return status;								// rckgabe des led status
}