/************************************************************************/
/* my_can.h                                                             */
/************************************************************************/
/* Musterlsung zum Praktikumsversuch CAN								*/
/*																		*/
/* Created:		16.07.2013 (SS2013)										*/
/* Author:		Nils Feldkmper											*/
/*				Jaron Martinez											*/
/************************************************************************/
#ifndef _MY_CAN_H
#define _MY_CAN_H
/************************************************************************/
/* Defines                                                              */
/************************************************************************/
// Konfigurieren welches Board verwendet wird
#define _BOARD      1

// Adressen setzen
#if(_BOARD == 1)
	#define _BOARD_ADDR_SEND_OFFSET		0x010
	#define _BOARD_ADDR_GET_OFFSET		0x020
#elif(_BOARD == 2)
	#define _BOARD_ADDR_SEND_OFFSET		0x020
	#define _BOARD_ADDR_GET_OFFSET		0x010
#endif

// Makros um die Adressen zu generieren
#define ID_SEND(id)		(_BOARD_ADDR_SEND_OFFSET | id)
#define ID_GET(id)		(_BOARD_ADDR_GET_OFFSET | id)

// Makros fr bit Operationen
#define getBit(var,bit)			((1<<(bit))&var)
#define getBitValue(var,bit)	(((var)>>(bit))&1)
#define setBit(var,bit)			(var)|=(1<<(bit))
#define clearBit(var,bit)		(var)&=~(1<<(bit))
#define toggleBit(var, bit)		if(getBit(var,bit)){ clearBit(var,bit); } else { setBit(var,bit); }
#define writeBit(var,bit,val)   if(val==1){ setBit(var,bit); } else { clearBit(var,bit); }
	
// Mobs festlegen
#define NO_MOBS 15
#define NOMOB   0xff		// NOMOB = 15

/************************************************************************/
/* Globale Variablen                                                    */
/************************************************************************/
// CAN2.0 A message
typedef struct
{
	uint32_t 	id;         // ID
	uint8_t 	length;   	// Datenlnge
	uint8_t 	data[8];	// Daten
	uint8_t 	new_data;  	// sind neue Daten enthalten? "0"=Nein, "1"=Ja
}CAN_message;

CAN_message 	message_RX_global;						// Speichert die empfangene CAN-Nachricht

//Structs, um die Maskierungen und Identifier der einzelnen MOBs aufzunehmen
typedef struct
{
	uint32_t 	id;			//Identifier, auf den verglichen werden soll
	uint32_t 	mask;		//Maskierung, mit dem die eingehenden IDs maskiert werden
	uint8_t 	length;		//Lnge der Botschaft
	uint8_t 	direction;	// 00 - disable
							// 01 - enable transmission
							// 10 - enable reception
							// 11 - enable frame buffer reception
}MOB_def;

MOB_def 		MOB_all[14];							// Definiton der einzelnen MOb's

/************************************************************************/
/* Funktionen                                                           */
/************************************************************************/
void			MOB_init();								// Initialisierung der einzelnen MOBs
void 			MOB_Config();							// Die einzelnen MOBs mit der Hilfe der CANPAGE konfigurieren.
uint8_t 		getmob( uint32_t bitmask);				// Get idex of MOb for which an interrupt was detected.
void 			can_init( void);						// CAN initialization
void 			can_tx (CAN_message msg, uint8_t mob);	// CAN transmission via mailbox
uint8_t			CAN_getMsg(uint8_t status);				// Auf neue Nachrichten reagieren
CAN_message* 	get_RX_message(void);					// Empfangene CAN-Nachricht lesen

#endif