/************************************************************************/
/* my_io.c                                                              */
/************************************************************************/
/* Musterlsung zum Praktikumsversuch CAN								*/
/*																		*/
/* Created:		16.07.2013 (SS2013)										*/
/* Author:		Nils Feldkmper											*/
/*				Jaron Martinez											*/
/************************************************************************/

/************************************************************************/
/* Include                                                              */
/************************************************************************/
#include <avr/io.h>			// IO Register einbinden
#include <avr/interrupt.h>	// Interrupts einbinden
#include "my_io.h"			// Controller IO einbinden

/************************************************************************/
/* Globale Variablen                                                    */
/************************************************************************/
extern uint8_t but_freigabe;
extern uint8_t but_ticks;

/************************************************************************/
/* Funktionen                                                           */
/************************************************************************/
// **** init_ports() ****
void init_ports() 
{
	// **** PortA ****
	DDRA 	= 0xff;	// PortA auf Ausgang setzen (Anschluss der LEDs)
	PORTA	= 0x00;	// alle Ausgnge auf LOW (LEDs Active high)


	//**** PortE ****
	DDRE	= 0x00;	// PortE auf Eingang setzen (Taster)
	PORTE	= 0xff;	// interne Pull-Up-Widerstnde aktivieren
}

// **** setLEDS() ****
void setLED(uint8_t led) 
{
	PORTA |= (1 << led);	// LED einschalten (Bit setzen)
}

// **** clearLED() ****
void clearLED(uint8_t led) 
{
	PORTA &= ~(1 << led);	// LED ausschalten (Bit lschen) 
}

// **** toggleLED ****
void toggleLED(uint8_t led) 
{
	PORTA ^= (1 << led);	// LED togglen (Bit togglen) 
}

// **** Taster-Funktionen ****
uint8_t getTaster(uint8_t taster) 
{
	if (but_freigabe == 1) // wenn Taster freigegeben
	{			
		if (!(PINE & (1 << taster)))	// wenn gewhlter Taster gedrckt
		{	
			but_freigabe = 0;	// Taster sperren
			return 1;			// TRUE zurckgeben
		}
	}
	return 0;	// ansonsten 0 zurckgeben

}
		
/*	zum Entprellen: 250ms Sperre reichen, also 125/4 = 31.25 = 31 Ticks */
// **** ISR Timer2 CompareMatch ****
ISR (TIMER0_COMP_vect) 
{
	if (but_freigabe != 1)	// wenn Taster gesperrt
	{	
		but_ticks--;		// Entprellzeit dekrementieren
	}

	if (but_ticks == 0)		// Entprellzeit abgelaufen
	{		
		but_freigabe = 1;	// Tasterfreigabe setzen
		but_ticks	 = 31;	// Entprellzeit neu vorladen
	}
}


// **** init_timer0() ****
void init_timer0() 
{
	// Berechnung des Prescalers:
	// Systemtakt 8MHz, Prescaler 256 -> 31250 Ticks pro Sekunde
	// (Ticks pro Sekunde) / OCR0A = (Anzahl der Interrupts pro Sekunde)
	// so haben wir die exakte Zeitbasis fr eine Sekunde
	// fr OCR0A = 250:
	// 31250/250 = 125 Interrupts pro Sekunde (in ISR abfragen)
	// Prescaler, bei denen eine Kommazahl als Ergebnis kommt (zB Prescaler 1024) sind ungeeignet bzw es 
	// ist aufwendiger das zu berechnen
	// zum Entprellen: 250ms Sperre reichen, also 125/4 = 31.25 = 31 Ticks

	TCCR0A	= (1 << WGM01) | 
			  (1 << CS02)   ;		// Timer2 konfigurieren

	TIMSK0	= (1 << OCIE0A);		// CompareMatch Interrupt aktivieren

	OCR0A	= 250 -1;				// Comparewert (-1, fr Fehlerausgleich)
}